// log files generated by
//
// git log --name-only --pretty=format:""
//
// NOTE: use git-also.js if you wan to run this in the git repository
var readline = require('readline');
var fs = require('fs');
var computeSimilarities = require('./lib/computeSimilarities.js');
var startShell = require('./lib/startShell.js');

var inputFileName = process.argv[2];

if (!fs.existsSync(inputFileName)) {
  console.log('Cannot find input file.');
  console.log('');
  console.log('To create input file, run the following command: ');
  console.log('');
  console.log('git log --name-only --pretty=format:"" > inputFile.log');
  console.log('');
  console.log('Then re-run this program: ');
  console.log('');
  console.log('node index.js inputFile.log');
  process.exit(1)
}

console.log('reading file: ' + inputFileName);

var rl = readline.createInterface({
  input: fs.createReadStream(inputFileName)
});

var buffer = [];
var commits = [];

rl.on('line', processLine);
rl.on('close', startInteractiveShell);

function processLine(line) {
  if (line) {
    buffer.push(line)
  } else {
    if (buffer.length > 0) {
      commits.push(buffer);
      buffer = [];
    }
  }
}

function startInteractiveShell() {
  var similarities = computeSimilarities(commits)
  startShell(similarities);
}
